/*! \file ds1621.c \brief Obsluga termometru DS1621 przez I2C */
/*
 *   Oprogramowanie Robota
 *   Michal Slomkowski m.slomkowski@interia.pl, www.flylab.glt.pl
 */

#include <inttypes.h>
#include <avr/io.h>
#include "i2c.h"
#include "ds1621.h"
#include "harddef.h"
#include <util/delay.h>

// DS1621 Standalone Command
// Returns 1 if OK
static int ds1621_standalone_cmd(BYTE ucDevice, BYTE ucCmd)
{
	uint8_t iRetVal = 0;
	if(!i2c_start()) return 0;

	if((i2c_select_write(ucDevice)) && (i2c_write(&ucCmd, 1))) iRetVal = 1;
	i2c_stop();

	return iRetVal;
}

// DS1621 Write to Register
// Returns 1 if OK
static int ds1621_write_reg(BYTE ucDevice, BYTE ucCmd, BYTE* ucReg, BYTE ucLen)
{
	int iRetVal = 0;

	if(!i2c_start()) return 0;

	if((i2c_select_write(ucDevice)) && (i2c_write(&ucCmd, 1)) && 
	(i2c_write(ucReg, ucLen))) iRetVal = 1;

	i2c_stop();

	return iRetVal;
}

// DS1621 Read from Register
// Returns 1 if OK
static int ds1621_read_reg(BYTE ucDevice, BYTE ucCmd, BYTE* ucReg, BYTE ucLen)
{
	int iRetVal = 0;

	if(!i2c_start()) return 0;

	if((i2c_select_write(ucDevice)) && (i2c_write(&ucCmd, 1)) && (i2c_start()) &&
		(i2c_select_read(ucDevice)) && (i2c_read(ucReg, ucLen))) iRetVal = 1;
	i2c_stop();

	return iRetVal;
}

// DS1621 Read Temperature
// Returns 1 if OK
int ds1621_read_temp(BYTE ucDevice, BYTE* ucTemp)
{
	return ds1621_read_reg(ucDevice, DS1621_READTEMP, ucTemp, 2);
}

// DS1621 Set Configuration
// Returns 1 if OK
int ds1621_set_config(BYTE ucDevice, BYTE ucConfig)
{
	return ds1621_write_reg(ucDevice, DS1621_SETCONFIG, &ucConfig, 1);
}

// DS1621 Read Configuration
// Returns 1 if OK
int ds1621_read_config(BYTE ucDevice, BYTE* ucConfig)
{
	return ds1621_read_reg(ucDevice, DS1621_SETCONFIG, ucConfig, 1);
}

// DS1621 Start Conversion
// Returns 1 if OK
int ds1621_start_convert(BYTE ucDevice)
{
	return ds1621_standalone_cmd(ucDevice, DS1621_STARTCONVERT);
}

// DS1621 Stop Conversion
// Returns 1 if OK
int ds1621_stop_convert(BYTE ucDevice)
{
	return ds1621_standalone_cmd(ucDevice, DS1621_STOPCONVERT);
}

int ds1621_temperatura(void)
{
	BYTE ucTemp[2];
	
	if(!(ds1621_set_config(DS1621_ADDR, BV(DS1621_CONF_1SHOT)))) return -300;
	ds1621_start_convert(DS1621_ADDR); // Start conversion

	_delay_ms(200);// Wait some time, till the conversion is finished

	ds1621_read_temp(DS1621_ADDR, ucTemp);

	return ucTemp[0]; // zwraca pierwszy bajt - temperatura w stopniach C
}


