/*! \file i2c.c \brief Obsluga magistrali I2C (TWI) */
/*
 *   Oprogramowanie Robota
 *   Michal Slomkowski m.slomkowski@interia.pl, www.flylab.glt.pl
 */

#include <avr/io.h>
#include <inttypes.h>

#include "harddef.h"
#include "i2c.h"

void i2cInit(void)
{
	uint8_t bitrate_div;
	// set pull-up resistors on I2C bus pins
	sbi(PORTC, PC5);
	sbi(PORTC, PC4);

	// set i2c bit rate to 100KHz
	bitrate_div = ((F_CPU/1000l)/100);
	if(bitrate_div >= 16) bitrate_div = (bitrate_div-16)/2;
	outb(TWBR, bitrate_div);
	
	// enable TWI (two-wire interface)
	sbi(TWCR, TWEN);
	
	// enable TWI interrupt and slave address ACK
	sbi(TWCR, TWIE);
	sbi(TWCR, TWEA);
}

// I2C Start
// Returns 1 if OK
uint8_t i2c_start(void)
{
	TWCR = _BV(TWINT) | _BV(TWSTA) | _BV(TWEN);  // Send start condition
	while(!(TWCR & _BV(TWINT)));  // Wait
	return((TWSR & 0xF8) == 0x08 || (TWSR & 0xF8) == 0x10);  // Return status
}

// I2C Stop
void i2c_stop(void)
{
	TWCR = _BV(TWINT) | _BV(TWEN) | _BV(TWSTO);  // Send stop condition
}

// I2C select device and read mode
// Returns 1 if OK
int i2c_select_read(BYTE ucDevice)
{
	TWDR = (ucDevice << 1) | 0x01;
	TWCR = _BV(TWINT) | _BV(TWEN);  // Begin transmission of address
	while(!(TWCR & _BV(TWINT)));  // Wait
	return((TWSR & 0xF8) == 0x40);  // Return status
}

// I2C select device and write mode
// Returns 1 if OK
int i2c_select_write(BYTE ucDevice)
{
	TWDR = ucDevice << 1;
	TWCR = _BV(TWINT) | _BV(TWEN);  // Begin transmission of address
	while(!(TWCR & _BV(TWINT)));  // Wait
	return((TWSR & 0xF8) == 0x18);  // Return status
}

// I2C write bytes
// Returns 1 if OK
int i2c_write(BYTE* ucData, BYTE ucLen)
{
	while(ucLen)
	{
		ucLen--;
		TWDR = *ucData;
		ucData++;
		TWCR = _BV(TWINT) | _BV(TWEN);// Begin transmission of data

		while(!(TWCR & _BV(TWINT)));// Wait

		if((TWSR & 0xF8) != 0x28) return 0;
	}
	return 1;
}

// I2C read bytes
// Returns 1 if OK
int i2c_read(BYTE* ucData, BYTE ucLen)
{
	while(ucLen)
	{
		ucLen--;
		if(ucLen)
		{
			TWCR = _BV(TWINT) | _BV(TWEN) | _BV(TWEA);// Enable new incoming data, ack
		}
		else
		{
			TWCR = _BV(TWINT) | _BV(TWEN);// Enable new incoming data, nack (last byte)
		}

		while(!(TWCR & _BV(TWINT)));// Wait

		switch(TWSR & 0xF8)
		{
			case 0x50:
				*ucData = TWDR;
				ucData++;
				break;
			case 0x58:
				if(ucLen) return 0;// Error
				else
				{
					*ucData = TWDR;
					ucData++;
				}
				break;
			default:
				return 0;
		};
	}
	return 1;
}
